from sqlalchemy import *

import config

from globals import logger

from utils import *
from consts import *
from sqlManager import *

### Analysis module ###
# This module is for functions used for
# analysing table data after executing main query

def getRunRepackedPercentage(runid, numStreamers):
    """
    Get how many percent streamers files were repacked
    """
    assocTable = Tbl(TABLES.REPACK_STREAMER_ASSOC)
    repackedTable = Tbl(TABLES.REPACKED)

    query = select([func.count(func.distinct(assocTable.c.streamer_id))],
                        repackedTable.c.run_id == runid,
                        from_obj=[
                            repackedTable.join(assocTable, repackedTable.c.repacked_id == assocTable.c.repacked_id)
                        ])
    result = executeQuery(query)
    numRepacked = int(result.fetchone()[0])

    if (numStreamers == 0):
        percentage = 100
    else:
        percentage = numRepacked * 100 / numStreamers
    return Percentage(percentage, numRepacked)

def getRunRepackMergePercentage(runid, numRepacked):
    """
    Get how many percent repacked files were merged
    """
    repackedTable = Tbl(TABLES.REPACKED)
    parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)

    numMerged = 0

    query = select([func.count(repackedTable.c.repacked_id)],
        repackedTable.c.run_id == runid,
        from_obj = [
            repackedTable.join(parentageTable, repackedTable.c.repacked_id == parentageTable.c.input_id)
        ]
    )
    result = executeQuery(query)
    numMerged += int(result.fetchone()[0])

    query = select([func.count(repackedTable.c.repacked_id)],
        and_(
            repackedTable.c.run_id == runid,
            not_(
                repackedTable.c.repacked_id.in_(
                select([parentageTable.c.output_id])
            )),
            or_(
                repackedTable.c.status == REPACK_RECONSTRUCTION_DISABLED,
                repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED
            )
        )
    )
    result = executeQuery(query)
    numMerged += int(result.fetchone()[0])

    if numRepacked == 0:
        percentage = 100
    else:
        percentage = numMerged * 100 / numRepacked

    return Percentage(percentage, numMerged)

def getRunRecoPercentage(runid, numReconstructed):
    """
    Get how many merged files, marked for reconstruction were reconstructed
    """
    repackedTable = Tbl(TABLES.REPACKED)

    query = select([func.count(repackedTable.c.repacked_id)],
                        and_(
                            repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED,
                            repackedTable.c.run_id == runid
                        )
                    )
    result = executeQuery(query)
    numRepacked = int(result.fetchone()[0])

    if (numRepacked == 0):
        percentage = 100
    else:
        percentage = numReconstructed * 100 / numRepacked
    return Percentage(percentage, numReconstructed)

class Percentage:
    def __init__(self, percentage, value = None, total = None):
        """
        Parameters:

        percentage -    percentage
        value -         value for percentage
        total -         how much is 100%
        """
        self.percentage = percentage
        self.value = value
        self.total = total
